/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InterceptorsClassResolver.java 400 2006-04-24 16:38:38Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.helper.bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.asm.Type;

import org.objectweb.easybeans.deployment.annotations.InterceptorType;
import org.objectweb.easybeans.deployment.annotations.JClassInterceptor;
import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.exceptions.ResolverException;
import org.objectweb.easybeans.deployment.annotations.impl.JInterceptors;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;
import org.objectweb.easybeans.naming.interceptors.ENCManager;

/**
 * This class sets the EasyBeans interceptors used when invoking business methods and also for life cycle events.
 * @author Florent Benoit
 */
public final class InterceptorsClassResolver {

    /**
     * Signature of EasyBeans interceptors.
     */
    private static final JMethod EASYBEANS_INTERCEPTOR = new JMethod(0, "intercept",
            "(Lorg/objectweb/easybeans/api/EasyBeansInvocationContext;)Ljava/lang/Object;",
            null, new String[] {"java/lang/Exception"});


    /**
     * Helper class, no public constructor.
     */
    private InterceptorsClassResolver() {

    }


    /**
     * Found all interceptors of the class (including business and lifecycle events) and also set EasyBeans interceptors.
     * @param classAnnotationMetadata class to analyze
     * @throws ResolverException if metadata is missing
     */
    public static void resolve(final ClassAnnotationMetadata classAnnotationMetadata) throws ResolverException {

        // First, EasyBeans interceptors
        List<String> easyBeansInterceptorsClasses = new ArrayList<String>();
        // Add EasyBeans interceptors for all methods
        easyBeansInterceptorsClasses.add(Type.getInternalName(ENCManager.getInterceptorClass()));
        // Set list of global interceptors (applied on all business methods)
        List<JClassInterceptor> easyBeansGlobalInterceptors = new ArrayList<JClassInterceptor>();
        for (String easyBeansInterceptor : easyBeansInterceptorsClasses) {
            easyBeansGlobalInterceptors.add(new JClassInterceptor(easyBeansInterceptor, EASYBEANS_INTERCEPTOR));
        }
        classAnnotationMetadata.setGlobalEasyBeansInterceptors(easyBeansGlobalInterceptors);

        // TODO: adds the default interceptor (set in the DD)


        // And then, set the user interceptors (found in the class)
        List<String> beanInterceptorsClasses = new ArrayList<String>();

        // Interceptors from other classes (will be analyzed after)
        // See 3.5.3 of simplified EJB 3.0 spec : multiple interceptors
        JInterceptors classIinterceptors = classAnnotationMetadata.getAnnotationInterceptors();
        if (classIinterceptors != null) {
            for (String cls : classIinterceptors.getClasses()) {
                beanInterceptorsClasses.add(cls);
            }
        }

        // interceptor in the bean class ? (LifeCycle event interceptors are not in the bean
        // because they don't take an InvocationContext as parameter, this is the intercepted method
        if (classAnnotationMetadata.getAroundInvokeMethodMetadata() != null) {
            beanInterceptorsClasses.add(classAnnotationMetadata.getClassName());
        }

        // Get the interceptors defined by the user on the class
        Map<InterceptorType, List<JClassInterceptor>> mapClassInterceptors =
            getInterceptors(classAnnotationMetadata.getClassName(), classAnnotationMetadata.getEjbJarAnnotationMetadata(),
                    beanInterceptorsClasses);
        classAnnotationMetadata.setUserInterceptors(mapClassInterceptors);


        // Now, analyze each interceptors found on methods.
        for (MethodAnnotationMetadata methodAnnotationMetaData
                : classAnnotationMetadata.getMethodAnnotationMetadataCollection()) {
            JInterceptors methodAnnotationInterceptors = methodAnnotationMetaData.getAnnotationInterceptors();
            if (methodAnnotationInterceptors != null) {
                List<String> methodInterceptorsClasses = new ArrayList<String>();
                for (String cls : methodAnnotationInterceptors.getClasses()) {
                    methodInterceptorsClasses.add(cls);
                }
                Map<InterceptorType, List<JClassInterceptor>> mapMethodInterceptors =
                    getInterceptors(classAnnotationMetadata.getClassName() + "/Method "
                            + methodAnnotationMetaData.getMethodName(),
                            classAnnotationMetadata.getEjbJarAnnotationMetadata(), methodInterceptorsClasses);
                methodAnnotationMetaData.setUserInterceptors(mapMethodInterceptors);
            }

        }

    }


    /**
     * Found interceptors method in the given class. It will analyze each interceptor class and
     * fill a structure with a mapping between the annotation type and the corresponding interceptors.
     * @param referencingName name of the class/method that reference these interceptors
     * @param ejbJarAnnotationMetadata root of all annotations (used to find the metadata)
     * @param interceptorsClasses list of classes that contains interceptors
     * @return the map between the type of interceptor (PostConstrut, AroundInvoke, ...) and the JClassInterceptor objects
     * @throws ResolverException if analyze fails
     */
    private static Map<InterceptorType, List<JClassInterceptor>> getInterceptors(final String referencingName,
            final EjbJarAnnotationMetadata ejbJarAnnotationMetadata,
            final List<String> interceptorsClasses) throws ResolverException {
        // Define the mapping object.
        Map<InterceptorType, List<JClassInterceptor>> mapInterceptors = new HashMap<InterceptorType, List<JClassInterceptor>>();
        // Init the map for each interceptor type
        for (InterceptorType type : InterceptorType.values()) {
            mapInterceptors.put(type, new ArrayList<JClassInterceptor>());
        }

        // For each interceptors classes, take the method with @AroundInvoke or @PostConstruct, etc. and build list
        for (String className : interceptorsClasses) {
            ClassAnnotationMetadata interceptorMetadata = ejbJarAnnotationMetadata.getClassAnnotationMetadata(className);
            if (interceptorMetadata == null) {
                throw new ResolverException("No medata for interceptor class " + className
                        + " referenced by " + referencingName);
            }

            // No inner class of the bean
            if (interceptorMetadata.getClassName().contains("$")) {
                throw new IllegalStateException("Interceptor can't be defined in an inner class.");
            }

            // Takes all methods of the super class and add them to the current class.
            InheritanceMethodResolver.resolve(interceptorMetadata);

            // Analyze methods of the interceptor meta-data and add it in the map
            for (MethodAnnotationMetadata method : interceptorMetadata.getMethodAnnotationMetadataCollection()) {
                // A method can be designed to run for all annotation, so no "else if" !
                if (method.isAroundInvoke()) {
                    mapInterceptors.get(InterceptorType.AROUND_INVOKE).add(new JClassInterceptor(className, method.getJMethod()));
                }
                if (method.isPostActivate()) {
                    mapInterceptors.get(InterceptorType.POST_ACTIVATE).add(new JClassInterceptor(className, method.getJMethod()));
                }
                if (method.isPostConstruct()) {
                    mapInterceptors.get(InterceptorType.POST_CONSTRUCT).add(new JClassInterceptor(className,
                            method.getJMethod()));
                }
                if (method.isPreDestroy()) {
                    mapInterceptors.get(InterceptorType.PRE_DESTROY).add(new JClassInterceptor(className, method.getJMethod()));
                }
                if (method.isPrePassivate()) {
                    mapInterceptors.get(InterceptorType.PRE_PASSIVATE).add(new JClassInterceptor(className, method.getJMethod()));
                }

            }

        }
        return mapInterceptors;
    }


}
