/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassAnnotationMetadata.java 373 2006-04-21 08:20:44Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.metadata;

import static javax.ejb.TransactionAttributeType.REQUIRED;
import static javax.ejb.TransactionManagementType.CONTAINER;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.objectweb.easybeans.deployment.annotations.ClassType;
import org.objectweb.easybeans.deployment.annotations.InterceptorType;
import org.objectweb.easybeans.deployment.annotations.JClassInterceptor;
import org.objectweb.easybeans.deployment.annotations.JField;
import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.exceptions.InterceptorsValidationException;
import org.objectweb.easybeans.deployment.annotations.impl.JAnnotationResource;
import org.objectweb.easybeans.deployment.annotations.impl.JCommonBean;
import org.objectweb.easybeans.deployment.annotations.impl.JEjbEJB;
import org.objectweb.easybeans.deployment.annotations.impl.JInterceptors;
import org.objectweb.easybeans.deployment.annotations.impl.JLocal;
import org.objectweb.easybeans.deployment.annotations.impl.JMessageDriven;
import org.objectweb.easybeans.deployment.annotations.impl.JRemote;
import org.objectweb.easybeans.deployment.annotations.impl.JStateful;
import org.objectweb.easybeans.deployment.annotations.impl.JStateless;
import org.objectweb.easybeans.deployment.annotations.metadata.interfaces.IEJBInterceptors;
import org.objectweb.easybeans.deployment.annotations.metadata.interfaces.ITransactionAttribute;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;


/**
 * This class represents the annotation metadata of a Bean.<br>
 * From this class, we can access to all methods of a bean with its associated information.
 * @author Florent Benoit
 */
public class ClassAnnotationMetadata extends CommonAnnotationMetadata implements ITransactionAttribute, IEJBInterceptors {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(ClassAnnotationMetadata.class);

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, MethodAnnotationMetadata> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, FieldAnnotationMetadata> fieldsAnnotationMetadata = null;

    /**
     * Parent meta data.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * List of local interfaces.
     */
    private JLocal jLocal = null;

    /**
     * List of remote interfaces.
     */
    private JRemote jRemote = null;

    /**
     * CommonBean description.
     */
    private JCommonBean jCommonBean = null;

    /**
     * Message Driven attribute.
     */
    private JMessageDriven jMessageDriven = null;

    /**
     * Stateless attribute.
     */
    private JStateless jStateless = null;

    /**
     * Stateful attribute.
     */
    private JStateful jStateful = null;


    /**
     * Local Home.
     */
    private String localHome = null;

    /**
     * Remote Home.
     */
    private String remoteHome = null;

    /**
     * List of annotation interceptors.
     */
    private JInterceptors annotationInterceptors = null;

    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors
     * that need to be present first on all methods.
     */
    private List<JClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in its bean class.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> userInterceptors = null;

    /**
     * Transaction management type (default = container).
     */
    private TransactionManagementType transactionManagementType = CONTAINER;

    /**
     * Transaction attribute type (default = required).
     */
    private TransactionAttributeType transactionAttributeType = REQUIRED;

    /**
     * Application exception annotation.
     */
    private ApplicationException applicationException = null;

    /**
     * Superclass name.
     */
    private String superName = null;

    /**
     * Interfaces of this clas.
     */
    private String[] interfaces = null;

    /**
     * The type of the class.
     * @see ClassType
     */
    private ClassType classType = null;

    /**
     * Name of the class associated to this metadata.
     */
    private String className = null;

    /**
     * List of &#64;{@link javax.ejb.AroundInvoke} methods on this class (should be only one per class, validating occurs after).
     */
    private List<MethodAnnotationMetadata> aroundInvokeMethodMetadatas = null;

    /**
     * Object representing &#64;{@link javax.ejb.EJBs} annotation.
     */
    private List<JEjbEJB> jEjbEJBs = null;

    /**
     * Object representing &#64;{@link javax.annotation.Resources} annotation.
     */
    private List<JAnnotationResource> jAnnotationResources = null;

    /**
     * Method used for &#64;{@link javax.annotation.PostConstruct} on this class (only one per class).
     */
    private MethodAnnotationMetadata postConstructMethodMetadata = null;

    /**
     * Method used for &#64;{@link javax.annotation.PreDestroy} on this class (only one per class).
     */
    private MethodAnnotationMetadata preDestroyMethodMetadata = null;

    /**
     * Method used for &#64;{@link javax.ejb.PostActivate} on this class (only one per class).
     */
    private MethodAnnotationMetadata postActivateMethodMetadata = null;

    /**
     * Method used for &#64;{@link javax.ejb.PrePassivate} on this class (only one per class).
     */
    private MethodAnnotationMetadata prePassivateMethodMetadata = null;

    /**
     * Is that the class represented by this metadata has already been modified ?
     */
    private boolean modified = false;

    /**
     * Constructor.
     * @param className name of the class associated to these metadatas.
     * @param ejbJarAnnotationMetadata parent metadata object.
     */
    public ClassAnnotationMetadata(final String className, final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) {
        this.className = className;
        this.methodsAnnotationMetadata = new HashMap<JMethod, MethodAnnotationMetadata>();
        this.fieldsAnnotationMetadata = new HashMap<JField, FieldAnnotationMetadata>();
        this.ejbJarAnnotationMetadata = ejbJarAnnotationMetadata;
    }

    /**
     * @return name of the bean (associated to this metadata).
     */
    public String getClassName() {
        return className;
    }

    /**
     * Add method annotation metadata for a given Bean.
     * @param methodAnnotationMetadata metadata of a method.
     */
    public void addMethodAnnotationMetadata(final MethodAnnotationMetadata methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // already exists ?
        if (methodsAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("BeanAnnotationMetadata.addMethodAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    public MethodAnnotationMetadata getMethodAnnotationMetadata(final JMethod jMethod) {
        return methodsAnnotationMetadata.get(jMethod);
    }

    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    public Collection<MethodAnnotationMetadata> getMethodAnnotationMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }


    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    public void addFieldAnnotationMetadata(final FieldAnnotationMetadata fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("BeanAnnotationMetadata.addFieldAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    public FieldAnnotationMetadata getFieldAnnotationMetadata(final JField jField) {
        return fieldsAnnotationMetadata.get(jField);
    }

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    public Collection<FieldAnnotationMetadata> getFieldAnnotationMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }


    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    public void setLocalInterfaces(final JLocal jLocal) {
        this.jLocal = jLocal;
    }

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    public void setRemoteInterfaces(final JRemote jRemote) {
        this.jRemote = jRemote;
    }


    /**
     * @return the local interfaces of this class.
     */
    public JLocal getLocalInterfaces() {
       return jLocal;
    }

    /**
     * @return the remote interfaces of this class.
     */
    public JRemote getRemoteInterfaces() {
        return jRemote;
    }

      /**
     * @return true if the class is a stateless class
     */
    public boolean isStateless() {
        return (classType != null && classType == ClassType.STATELESS);
    }

    /**
     * @return true if the class is a stateful class
     */
    public boolean isStateful() {
        return (classType != null && classType == ClassType.STATEFUL);
    }

    /**
     * @return true if the class is a session bean class
     */
    public boolean isSession() {
        return (classType != null && (classType == ClassType.STATELESS || classType == ClassType.STATEFUL));
    }

    /**
     * @return true if the class is an MDB class
     */
    public boolean isMdb() {
        return (classType != null && classType == ClassType.MDB);
    }

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.objectweb.easybeans.deployment.annotations.ClassType
     */
    public void setClassType(final ClassType cType) {
        this.classType = cType;
    }

    /**
     * @return Message driven attribute.
     */
    public JMessageDriven getJMessageDriven() {
        return jMessageDriven;
    }

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    public void setJMessageDriven(final JMessageDriven messageDriven) {
        jMessageDriven = messageDriven;
    }


    /**
     * @return string representation.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Class name
        concatStringBuilder("className", className, sb);

        // superclass name
        concatStringBuilder("superName", superName, sb);

        // interfaces
        concatStringBuilder("interfaces", interfaces, sb);

        // classType
        concatStringBuilder("classType", classType, sb);

        // jLocal
        concatStringBuilder("jLocal", jLocal, sb);

        // aroundInvokeMethodMetadatas
        concatStringBuilder("aroundInvokeMethodMetadatas", aroundInvokeMethodMetadatas, sb);

        // jRemote
        concatStringBuilder("jRemote", jRemote, sb);

        // jMessageDriven
        concatStringBuilder("jMessageDriven", jMessageDriven, sb);

        // remoteHome
        concatStringBuilder("remoteHome", remoteHome, sb);

        // localHome
        concatStringBuilder("localHome", localHome, sb);

        // transactionManagementType
        concatStringBuilder("transactionManagementType", transactionManagementType, sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", transactionAttributeType, sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", annotationInterceptors, sb);

        // jEjbEJBs
        concatStringBuilder("jAnnotationEJBs", jEjbEJBs, sb);

        // jAnnotationResources
        concatStringBuilder("jAnnotationResources", jAnnotationResources, sb);

        // Methods
        for (MethodAnnotationMetadata methodAnnotationMetadata : getMethodAnnotationMetadataCollection()) {
            concatStringBuilder("methods", methodAnnotationMetadata, sb);
        }

        sb.append("]");
        return sb.toString();
    }

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    public String getRemoteHome() {
        return remoteHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    public void setRemoteHome(final String remoteHome) {
        this.remoteHome = remoteHome;
    }

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    public String getLocalHome() {
        return localHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    public void setLocalHome(final String localHome) {
        this.localHome = localHome;
    }

    /**
     * @return transaction management type from @see TransactionManagementType.
     */
    public TransactionManagementType getTransactionManagementType() {
        return transactionManagementType;
    }

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value.
     *      (BEAN, CONTAINER)
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        this.transactionManagementType = transactionManagementType;
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return transactionAttributeType;
    }

    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return object representing list of &#64;{@link javax.ejb.Interceptors}.
     */
    public JInterceptors getAnnotationInterceptors() {
        return annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;{@link javax.ejb.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final JInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    public ApplicationException getApplicationException() {
        return applicationException;
    }

    /**
     * Sets the object representing the &#64;{@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    public void setApplicationException(final ApplicationException applicationException) {
        this.applicationException = applicationException;
    }

    /**
     * @return true if the classs is a Bean
     */
    public boolean isBean() {
        return isStateless() || isStateful() || isMdb();
    }

    /**
     * @return array of interfaces name.
     */
    public String[] getInterfaces() {
        return interfaces;
    }

    /**
     * Sets the interfaces of this class.
     * @param interfaces name of interfaces.
     */
    public void setInterfaces(final String[] interfaces) {
        this.interfaces = interfaces;
    }

    /**
     * @return the super class name.
     */
    public String getSuperName() {
        return superName;
    }

    /**
     * Sets the super class name.
     * @param superName name of the super class.
     */
    public void setSuperName(final String superName) {
        this.superName = superName;
    }

    /**
     * @return parent metadata object.
     */
    public EjbJarAnnotationMetadata getEjbJarAnnotationMetadata() {
        return ejbJarAnnotationMetadata;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getUserEasyBeansInterceptors() {
        return userInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    public void setUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> userInterceptors) {
        this.userInterceptors = userInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    public List<JClassInterceptor> getGlobalEasyBeansInterceptors() {
        return globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<JClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return the method metadata with annotation &#64;{@link javax.ejb.AroundInvoke}.
     */
    public MethodAnnotationMetadata getAroundInvokeMethodMetadata() {
        if (aroundInvokeMethodMetadatas != null) {
            // first item
            return aroundInvokeMethodMetadatas.get(0);
        }
        return null;
    }


    /**
     * @return the list of methods metadata with annotation &#64;{@link javax.ejb.AroundInvoke}.
     */
    public List<MethodAnnotationMetadata> getAroundInvokeMethodMetadatas() {
        return aroundInvokeMethodMetadatas;
    }

    /**
     * Add a &#64;{@link javax.ejb.AroundInvoke} method of this class.
     * @param aroundInvokeMethodMetadata the method.
     */
    public void addAroundInvokeMethodMetadata(final MethodAnnotationMetadata aroundInvokeMethodMetadata) {
        if (aroundInvokeMethodMetadatas == null) {
            this.aroundInvokeMethodMetadatas = new ArrayList<MethodAnnotationMetadata>();
        }
        aroundInvokeMethodMetadatas.add(aroundInvokeMethodMetadata);
    }

    /**
     * @return the method metadata with annotation &#64;{@link javax.annotation.PostConstruct}.
     */
    public MethodAnnotationMetadata getPostConstructMethodMetadata() {
        return postConstructMethodMetadata;
    }

    /**
     * Sets the&#64;{@link javax.annotation.PostConstruct} method of this class.
     * @param postConstructMethodMetadata the method.
     */
    public void setPostConstructMethodMetadata(final MethodAnnotationMetadata postConstructMethodMetadata) {
        if (this.postConstructMethodMetadata != null) {
            throw new InterceptorsValidationException("Class " + getClassName()
                    + " has already a @PostConstruct method which is "
                    + this.postConstructMethodMetadata.getMethodName() + ", cannot set new method "
                    + postConstructMethodMetadata);
        }
        this.postConstructMethodMetadata = postConstructMethodMetadata;
    }


    /**
     * @return the method metadata with annotation &#64;{@link javax.annotation.PreDestroy}.
     */
    public MethodAnnotationMetadata getPreDestroyMethodMetadata() {
        return preDestroyMethodMetadata;
    }

    /**
     * Sets the&#64;{@link javax.annotation.PreDestroy} method of this class.
     * @param preDestroyMethodMetadata the method.
     */
    public void setPreDestroyMethodMetadata(final MethodAnnotationMetadata preDestroyMethodMetadata) {
        if (this.preDestroyMethodMetadata != null) {
            throw new InterceptorsValidationException("Class " + getClassName() + " has already a @PreDestroy method which is "
                    + this.preDestroyMethodMetadata.getMethodName() + ", cannot set new method " + preDestroyMethodMetadata);
        }
        this.preDestroyMethodMetadata = preDestroyMethodMetadata;
    }


    /**
     * @return the method metadata with annotation &#64;{@link javax.ejb.PostActivate}.
     */
    public MethodAnnotationMetadata getPostActivateMethodMetadata() {
        return postActivateMethodMetadata;
    }

    /**
     * Sets the&#64;{@link javax.ejb.PostActivate} method of this class.
     * @param postActivateMethodMetadata the method.
     */
    public void setPostActivateMethodMetadata(final MethodAnnotationMetadata postActivateMethodMetadata) {
        if (this.postActivateMethodMetadata != null) {
            //TODO : I18n
            throw new InterceptorsValidationException("Class " + getClassName() + " has already a @PostActivate method which is "
                    + this.postActivateMethodMetadata.getMethodName() + ", cannot set new method " + postActivateMethodMetadata);
        }
        this.postActivateMethodMetadata = postActivateMethodMetadata;
    }


    /**
     * @return the method metadata with annotation &#64;{@link javax.ejb.PrePassivate}.
     */
    public MethodAnnotationMetadata getPrePassivateMethodMetadata() {
        return prePassivateMethodMetadata;
    }

    /**
     * Sets the&#64;{@link javax.ejb.PrePassivate} method of this class.
     * @param prePassivateMethodMetadata the method.
     */
    public void setPrePassivateMethodMetadata(final MethodAnnotationMetadata prePassivateMethodMetadata) {
        if (this.prePassivateMethodMetadata != null) {
            //TODO : I18n
            throw new InterceptorsValidationException("Class " + getClassName() + " has already a @PrePassivate method which is "
                    + this.prePassivateMethodMetadata.getMethodName() + ", cannot set new method " + prePassivateMethodMetadata);
        }
        this.prePassivateMethodMetadata = prePassivateMethodMetadata;
    }


    /**
     * Is that this class is an interceptor class ?
     * @return true if it the case, else false.
     */
    public boolean isInterceptor() {
        return (aroundInvokeMethodMetadatas != null && aroundInvokeMethodMetadatas.size() > 0)
        || postConstructMethodMetadata != null || preDestroyMethodMetadata != null
        || prePassivateMethodMetadata != null || postActivateMethodMetadata != null;
    }


    /**
     * @return jEjbEJBs list representing &#64;{@link javax.ejb.EJBs} annotation.
     */
    public List<JEjbEJB> getJEjbEJBs() {
        return jEjbEJBs;
    }

    /**
     * Set JEjbEJBs object.
     * @param jEjbEJBs list representing javax.ejb.EJBs annotation.
     */
    public void setJEjbEJBs(final List<JEjbEJB> jEjbEJBs) {
        this.jEjbEJBs = jEjbEJBs;
    }


    /**
     * @return JAnnotationResources list representing &#64;{@link javax.annotation.Resources} annotation.
     */
    public List<JAnnotationResource> getJAnnotationResources() {
        return jAnnotationResources;
    }

    /**
     * Sets JAnnotationResources object.
     * @param jAnnotationResources list representing javax.annotation.Resources annotation.
     */
    public void setJAnnotationResources(final List<JAnnotationResource> jAnnotationResources) {
        this.jAnnotationResources = jAnnotationResources;
    }

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    public JCommonBean getJCommonBean() {
        return jCommonBean;
    }

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    public void setJCommonBean(final JCommonBean commonBean) {
        jCommonBean = commonBean;
    }

    /**
     * @return the attributes for a Stateful
     */
    public JStateful getJStateful() {
        return jStateful;
    }

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    public void setJStateful(final JStateful jStateful) {
        this.jStateful = jStateful;
    }

    /**
     * @return the attributes for a Stateless
     */
    public JStateless getJStateless() {
        return jStateless;
    }

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    public void setJStateless(final JStateless jStateless) {
        this.jStateless = jStateless;
    }

    /**
     * @return true if the class has been modified.
     */
    public boolean wasModified() {
        return modified;
    }

    /**
     * Defines that this class has been modified.
     */
    public void setModified() {
        this.modified = true;
    }


}
