/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaContextHelper.java 130 2006-03-06 17:58:43Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.helper;

import java.util.List;

import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.xml.struct.EJB3;
import org.objectweb.easybeans.deployment.xml.struct.EnterpriseBeans;
import org.objectweb.easybeans.deployment.xml.struct.Session;
import org.objectweb.easybeans.deployment.xml.struct.common.EnvEntry;
import org.objectweb.easybeans.naming.NamingManager;

/**
 * Builds a Context (java: context).
 * @author Florent Benoit
 */
public final class JavaContextHelper {

    /**
     * Utility class.
     */
    private JavaContextHelper() {

    }

    /**
     * Build a new context for the given bean.
     * @param bean the bean for which there is a need to build a context.
     * @return a java: context.
     * @throws JavaContextHelperException if environment cannot be built
     */
    public static Context build(final ClassAnnotationMetadata bean) throws JavaContextHelperException {
        Context javaCtx = null;
        try {
            javaCtx = NamingManager.getInstance().createEnvironmentContext(bean.getClassName());
        } catch (NamingException e) {
            throw new IllegalStateException("Cannot build a new environment");
        }

        String beanClassName = bean.getClassName().replace("/", ".");

        Context envCtx;
        try {
            envCtx = javaCtx.createSubcontext("comp/env");
        } catch (NamingException e) {
            throw new JavaContextHelperException("Cannot create subcontext", e);
        }

        // get DD of the bean.
        EJB3 ejb3 = bean.getEjbJarAnnotationMetadata().getEjb3();

        String beanName = bean.getJCommonBean().getName();
        if (ejb3 != null) {
            // Get env-entry of the given bean.
            EnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
            if (enterpriseBeans != null) {
                List<Session> sessionList = enterpriseBeans.getSessionList();
                for (Session session : sessionList) {
                    // Not the bean, continue
                    if (!beanClassName.equals(session.getEjbClass())
                            && (beanName != null && !beanName.equals(session.getEjbName()))) {
                        continue;
                    }

                    // bean is found, get env-entry
                    List<EnvEntry> envEntryList = session.getEnvEntryList();
                    for (EnvEntry envEntry : envEntryList) {
                        String name = envEntry.getEnvEntryName();
                        Object value = getEnvEntryValue(envEntry);
                        // if null, no entry in java:comp/env as specified chapter 15.4.1
                        if (value != null) {
                            try {
                                envCtx.rebind(name, value);
                            } catch (NamingException e) {
                                throw new JavaContextHelperException("Cannot bind element '" + name + "'.", e);
                            }
                        }
                    }

                }

            }

        }

        return javaCtx;
    }

    /**
     * Gets the value for a given env-entry.
     * @param envEntry the element representing env-entry.
     * @return the value associated to the given element.
     */
    private static Object getEnvEntryValue(final EnvEntry envEntry) {
        final String type = envEntry.getEnvEntryType();
        final String value = envEntry.getEnvEntryValue();

        Object returnedValue = null;

        if (type.equals(Boolean.class.getName())) {
            if ("true".equalsIgnoreCase(value)) {
                returnedValue = Boolean.TRUE;
            } else if ("false".equalsIgnoreCase(value)) {
                returnedValue = Boolean.FALSE;
            }
        } else if (type.equals(String.class.getName())) {
            returnedValue = value;
        } else if (type.equals(Integer.class.getName())) {
            if (value != null) {
                returnedValue = new Integer(value);
            }
        } else if (type.equals(Character.class.getName())) {
            if (value != null) {
                if (value.length() != 1) {
                    throw new IllegalStateException("The value '" + value
                            + "' is not a valid value for env-entry of type java.lang.Character.");
                }
                returnedValue = new Character(value.charAt(0));
            }
        } else if (type.equals(Double.class.getName())) {
            if (value != null) {
                returnedValue = new Double(value);
            }
        } else if (type.equals(Byte.class.getName())) {
            if (value != null) {
                returnedValue = new Byte(value);
            }
        } else if (type.equals(Short.class.getName())) {
            if (value != null) {
                returnedValue = new Short(value);
            }
        } else if (type.equals(Long.class.getName())) {
            if (value != null) {
                returnedValue = new Long(value);
            }
        } else if (type.equals(Float.class.getName())) {
            if (value != null) {
                returnedValue = new Float(value);
            }
        } else {
            throw new IllegalStateException(type + " is not a valid type for env-entry.");
        }
        return returnedValue;
    }
}
