/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BeanClassAdapter.java 258 2006-03-23 17:04:36Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.enhancer.bean;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodAdapter;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;
import org.objectweb.easybeans.api.Factory;
import org.objectweb.easybeans.api.bean.EasyBeansBean;
import org.objectweb.easybeans.api.bean.EasyBeansMDB;
import org.objectweb.easybeans.api.bean.EasyBeansSFSB;
import org.objectweb.easybeans.api.bean.EasyBeansSLSB;
import org.objectweb.easybeans.container.session.EasyBeansSessionContext;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.enhancer.CommonClassGenerator;
import org.objectweb.easybeans.enhancer.interceptors.EasyBeansInvocationContextGenerator;

/**
 * This class adds a bean interface to the parsed object.<br>
 * It also adds getEasyBeansFactory method defined in the EasyBeansBean interface.<br>
 * Stateless bean will have EasyBeansStatelessSessionBean interface, etc.
 * @author Florent Benoit
 */
public class BeanClassAdapter extends ClassAdapter implements Opcodes {

    /**
     * Metadata available by this adapter for a class.
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Constructor.
     * @param classAnnotationMetadata object containing all attributes of the
     *        class
     * @param cv the class visitor to which this adapter must delegate calls.
     */
    public BeanClassAdapter(final ClassAnnotationMetadata classAnnotationMetadata, final ClassVisitor cv) {
        super(cv);
        this.classAnnotationMetadata = classAnnotationMetadata;
    }

    /**
     * Visits the header of the class.
     * @param version the class version.
     * @param access the class's access flags (see
     *        {@link org.objectweb.asm.Opcodes}). This parameter also indicates
     *        if the class is deprecated.
     * @param name the internal name of the class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     * @param signature the signature of this class. May be <tt>null</tt> if
     *        the class is not a generic one, and does not extend or implement
     *        generic classes or interfaces.
     * @param superName the internal of name of the super class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     *        For interfaces, the super class is {@link Object}. May be
     *        <tt>null</tt>, but only for the {@link Object} class.
     * @param interfaces the internal names of the class's interfaces (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     *        May be <tt>null</tt>.
     */
    @Override
    public void visit(final int version, final int access, final String name, final String signature,
            final String superName, final String[] interfaces) {

        String[] newInterfaces = null;

        // Add new interface with bean
        if (classAnnotationMetadata.isBean()) {
            // copy old interfaces in the new array
            newInterfaces = new String[interfaces.length + 1];
            System.arraycopy(interfaces, 0, newInterfaces, 0, interfaces.length);

            int indexElement = newInterfaces.length - 1;

            // Add the right interface (SLSB, SFSB, MDB)
            if (classAnnotationMetadata.isStateless()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansSLSB.class);
            } else if (classAnnotationMetadata.isStateful()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansSFSB.class);
            } else if (classAnnotationMetadata.isMdb()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansMDB.class);
            } else {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansBean.class);
            }
        } else {
            newInterfaces = interfaces;
        }

        super.visit(version, access, name, signature, superName, newInterfaces);

    }
    /**
     * Visits a method of the class. This method <i>must</i> return a new
     * {@link MethodVisitor} instance (or <tt>null</tt>) each time it is
     * called, i.e., it should not return a previously returned visitor.
     *
     * @param access the method's access flags (see {@link Opcodes}). This
     *        parameter also indicates if the method is synthetic and/or
     *        deprecated.
     * @param name the method's name.
     * @param desc the method's descriptor (see {@link Type Type}).
     * @param signature the method's signature. May be <tt>null</tt> if the
     *        method parameters, return type and exceptions do not use generic
     *        types.
     * @param exceptions the internal names of the method's exception classes
     *        (see {@link Type#getInternalName() getInternalName}). May be
     *        <tt>null</tt>.
     * @return an object to visit the byte code of the method, or <tt>null</tt>
     *         if this class visitor is not interested in visiting the code of
     *         this method.
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {

        // if default constructor, then needs to add a call to create a new InterceptorManager
        if ("<init>".equals(name) && "()V".equals(desc)) {
            MethodVisitor mv =  super.visitMethod(access, name, desc, signature, exceptions);
            return new AddMethodConstructorAdapter(mv);
        }
        return super.visitMethod(access, name, desc, signature, exceptions);

    }


    /**
     * Visits the end of the class. This method, which is the last one to be
     * called, is used to inform the visitor that all the fields and methods of
     * the class have been visited.
     */
    @Override
    public void visitEnd() {
        super.visitEnd();

        // Adds the factory attribute and its getter/setter.
        CommonClassGenerator.addFieldGettersSetters(cv, classAnnotationMetadata.getClassName(),
                "easyBeansFactory", Factory.class);


        if (classAnnotationMetadata.isBean()) {
            // Adds interceptor manager
            CommonClassGenerator.addFieldGettersSetters(cv, classAnnotationMetadata.getClassName(),
                    "easyBeansInterceptorManager", "L" + classAnnotationMetadata.getClassName()
                            + EasyBeansInvocationContextGenerator.SUFFIX_INTERCEPTOR_MANAGER + ";");

            // Adds the sessionContext attribute and its getter/setter.
            if (classAnnotationMetadata.isSession()) {
                CommonClassGenerator.addFieldGettersSetters(cv, classAnnotationMetadata.getClassName(),
                        "easyBeansSessionContext", EasyBeansSessionContext.class);
            }
        }

    }

    /**
     * Adds an entry in the constructor of the bean.
     * It will initialize the interceptorManager.
     * @author Florent Benoit
     */
    public class AddMethodConstructorAdapter extends MethodAdapter {

        /**
         * Constructs a new AddMethodConstructorAdapter object.
         * @param mv the code visitor to which this adapter must delegate calls.
         */
        public AddMethodConstructorAdapter(final MethodVisitor mv) {
            super(mv);
        }

        /**
         * Adds instruction just after the start of the method code.
         * TODO: Analyze when call to super() constructor is done and add instruction after.
         */
        @Override
        public void visitCode() {
            super.visitCode();
            String clManager = classAnnotationMetadata.getClassName()
            + EasyBeansInvocationContextGenerator.SUFFIX_INTERCEPTOR_MANAGER;
            mv.visitVarInsn(ALOAD, 0);
            mv.visitTypeInsn(NEW, clManager);
            mv.visitInsn(DUP);
            mv.visitMethodInsn(INVOKESPECIAL, clManager, "<init>", "()V");
            mv.visitFieldInsn(PUTFIELD, classAnnotationMetadata.getClassName(), "easyBeansInterceptorManager", "L"
                    + clManager + ";");

        }

    }

}

