/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JNDILookupHelper.java 237 2006-03-20 16:14:01Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.injection;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Helper class for injecting an object in JNDI (ENC env).
 * @author Florent Benoit
 */
public final class JNDIBinderHelper {

    /**
     * Type of lookup available.
     */
    public enum JndiType {JAVA_COMP_ENV}

    /**
     * Comp prefix.
     */
    private static final String JAVA_COMP = "java:comp/";

    /**
     * ENV prefix.
     */
    private static final String JAVA_COMP_ENV = JAVA_COMP + "env";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(JNDIBinderHelper.class);



    /**
     * Utility class, no public constructor.
     */
    private JNDIBinderHelper() {

    }

    /**
     * Gets the context for a given name (java: , java:comp, etc).
     * @param name the name of the context to lookup
     * @return context found or null.
     */
    public static Context getContext(final String name) {
        InitialContext ictx;
        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            logger.error("Cannot instantiate an initial context", e);
            return null;
        }

        Object o = null;
        try {
            o = ictx.lookup(name);
        } catch (NamingException e) {
            logger.error("Cannot find the JNDI name {0}", name, e);
        }
        if (o == null) {
            logger.error("No object was found for JNDI name {0}", name);
        }
        Context ctx = null;
        if (o instanceof Context) {
            ctx = (Context) o;
        } else {
            logger.error("Object not instance of context. Object = {0}", o);
        }
        return ctx;
    }

    /**
     * Bind a JNDI name object in java:comp/env/.
     * @param name the name of the object to bind.
     * @param object the value of the object.
     */
    public static void bindEnvJndiName(final String name, final Object object) {
        try {
            getContext(JAVA_COMP_ENV).rebind(name, object);
        } catch (NamingException e) {
            logger.error("Cannot bind object {0} with name {1}", object, name, e);
        }
    }


}
