/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CommonsLoggerImpl.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.log;

import java.text.MessageFormat;

import org.apache.commons.logging.Log;

import org.objectweb.easybeans.i18n.I18n;

/**
 * Wrapping around a commons logger allowing to use internationalization and MessageFormat API.
 * @author Florent Benoit
 */
public class CommonsLoggerImpl implements JLog {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 1707614695786252792L;

    /**
     * Wrapped commons logging logger.
     */
    private Log wrapped;

    /**
     * Instance of I18n.
     */
    private I18n i18n = null;

    /**
     * Build a new logger with the wrapped logger.
     * @param wrapped the delegate logger.
     * @param i18n the internationalization object to use, if any.
     */
    public CommonsLoggerImpl(final Log wrapped, final I18n i18n) {
        this.wrapped = wrapped;
        this.i18n = i18n;
    }

    /**
     * <p>
     * Is debug logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than debug.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isDebugEnabled() {
        return wrapped.isDebugEnabled();
    }

    /**
     * <p>
     * Is error logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than error.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isErrorEnabled() {
        return wrapped.isErrorEnabled();
    }

    /**
     * <p>
     * Is fatal logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than fatal.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isFatalEnabled() {
        return wrapped.isFatalEnabled();
    }

    /**
     * <p>
     * Is info logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than info.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isInfoEnabled() {
        return wrapped.isInfoEnabled();
    }

    /**
     * <p>
     * Is trace logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than trace.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isTraceEnabled() {
        return wrapped.isDebugEnabled();
    }

    /**
     * <p>
     * Is warn logging currently enabled?
     * </p>
     * <p>
     * Call this method to prevent having to perform expensive operations (for
     * example, <code>String</code> concatenation) when the log level is more
     * than warn.
     * </p>
     * @return true if it is enabled, else false
     */
    public boolean isWarnEnabled() {
        return wrapped.isWarnEnabled();
    }

    /**
     * <p>
     * Log a message with trace log level.
     * </p>
     * @param message log this message
     */
    public void trace(final Object message) {
        wrapped.trace(message);
    }

    /**
     * <p>
     * Log a message with trace log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void trace(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isTraceEnabled()) {
            return;
        }
        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.trace(msg, t);
        } else {
            wrapped.trace(msg);
        }
    }

    /**
     * <p>
     * Log a message with debug log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void debug(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isDebugEnabled()) {
            return;
        }
        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.debug(msg, t);
        } else {
            wrapped.debug(msg);
        }
    }

    /**
     * <p>
     * Log a message with info log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void info(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isInfoEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.info(msg, t);
        } else {
            wrapped.info(msg);
        }

    }

    /**
     * <p>
     * Log a message with warn log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void warn(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isWarnEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.warn(msg, t);
        } else {
            wrapped.warn(msg);
        }

    }

    /**
     * <p>
     * Log a message with error log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void error(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isErrorEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.error(msg, t);
        } else {
            wrapped.error(msg);
        }

    }

    /**
     * <p>
     * Log a message with fatal log level.
     * </p>
     * @param message - This could be
     *        <ul>
     *        <li>an entry of an I18n repository</li>
     *        <li>a preformated message given to MessageFormat class</li>
     *        <li>or a simple object on which toString() method will be called</li>
     *        </ul>
     * @param args could be empty or contains the object for the formatter (I18n
     *        case). To log an exception, the exception must be the last
     *        argument.
     */
    public void fatal(final Object message, final Object... args) {
        // Do nothing is level is not enabled
        if (!wrapped.isFatalEnabled()) {
            return;
        }

        String msg = getMessage(message, args);
        Throwable t = getThrowable(args);
        if (t != null) {
            wrapped.fatal(msg, t);
        } else {
            wrapped.fatal(msg);
        }

    }

    /**
     * Gets the i18n object associated to this logger.
     * @return i18n object.
     */
    public I18n getI18n() {
        return i18n;
    }


    /**
     * Gets a string message build with the given args.<br> *
     * @param message If the message is an i18n entry, this entry will be used
     *        to format the message.<br>
     *        Else, the arguments will be used as string formatter
     * @param args given args use by the formatter
     * @return a string representation of the given message.
     */
    private String getMessage(final Object message, final Object... args) {

        // I18n present ?
        if (i18n != null) {
            // try to format message
            return i18n.getMessage(String.valueOf(message), args);
        }

        // else, try to format message
        return MessageFormat.format(String.valueOf(message), args);
    }

    /**
     * Try to find a Throwable object in the array of object args. The throwable
     * should be the last parameter.
     * @param args array of objects on which we need to find the Throwable
     * @return a throwable object if last index if a Throwable or null
     */
    private Throwable getThrowable(final Object... args) {
        int size = args.length;
        // Detect the size
        if (size > 0) {
            Object lastArg = args[size - 1];
            if (lastArg instanceof Throwable) {
                return (Throwable) lastArg;
            }
        }

        // not found
        return null;
    }

}
