/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JPersistenceUnitInfoHelper.java 369 2006-04-21 08:18:35Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.persistence.xml;

import java.net.URL;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Class used to fill PersistenceInfo implementation class.
 * @author Florent Benoit
 */
public final class JPersistenceUnitInfoHelper {

    /**
     * Hibernate persistence provider.
     */
    private static final String HIBERNATE_PERSISTENCE_PROVIDER = "org.hibernate.ejb.HibernatePersistence";

    /**
     * Hibernate TM lookup.
     */
    private static final String HIBERNATE_TM_LOOKUP = "hibernate.transaction.manager_lookup_class";

    /**
     * Hibernate / JOTM transaction manager.
     */
    private static final String HIBERNATE_JOTM = "org.hibernate.transaction.JOTMTransactionManagerLookup";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(JPersistenceUnitInfoHelper.class);

    /**
     * Utility class, no public constructor.
     */
    private JPersistenceUnitInfoHelper() {

    }

    /**
     * Parses the given XML and complete the PersistenceUnitInfos structure
     * before returning it.
     * @param url the URL of the the Reader of the XML file.
     * @return array of jPersistenceInfo which are implementation of
     *         PersistenceUnitInfo object
     * @throws JPersistenceUnitInfoException if values are incorrect.
     */
    public static JPersistenceUnitInfo[] getPersistenceUnitInfo(final URL url) throws JPersistenceUnitInfoException {

        JPersistenceUnitInfo[] jPersistenceunitInfos = JPersistenceUnitInfoLoader.loadPersistenceUnitInfoImpl(url);

        for (JPersistenceUnitInfo jPersistenceunitInfo : jPersistenceunitInfos) {
            // analyze jta datasource
            String jtaDsName = jPersistenceunitInfo.getJtaDataSourceName();

            // try to get this datasource (if not null)
            if (jtaDsName != null && !jtaDsName.equals("")) {
                DataSource ds = null;
                try {
                    ds = (DataSource) new InitialContext().lookup(jtaDsName);
                } catch (NamingException e) {
                    // TODO: Remove
                    // try with jdbc_1
                    try {
                        logger.warn("Datasource named '" + jtaDsName
                                + "' was not found, use instead the default jndi name jdbc_1");
                        ds = (DataSource) new InitialContext().lookup("jdbc_1");
                    } catch (NamingException ne) {
                        throw new JPersistenceUnitInfoException("Cannot get jta DataSource with the JNDI name '"
                            + jtaDsName + "'.", ne);
                    }
                }
                jPersistenceunitInfo.setJtaDataSource(ds);
            }

            // analyze non jta datasource
            String nonJtaDsName = jPersistenceunitInfo.getNonJtaDataSourceName();
            // try to get this datasource (if not null)
            if (nonJtaDsName != null && !nonJtaDsName.equals("")) {
                DataSource ds = null;
                try {
                    ds = (DataSource) new InitialContext().lookup(nonJtaDsName);
                } catch (NamingException e) {
                    // TODO: Remove
                    // try with jdbc_1
                    try {
                        logger.warn("Datasource named '" + jtaDsName
                                + "' was not found, use instead the default jndi name jdbc_1");
                        ds = (DataSource) new InitialContext().lookup("jdbc_1");
                    } catch (NamingException ne) {
                        throw new JPersistenceUnitInfoException("Cannot get non jta DataSource with the JNDI name '"
                                + nonJtaDsName + "'.", ne);
                    }
                }
                jPersistenceunitInfo.setNonJtaDataSource(ds);
            }

            // Persistence Provider
            if (jPersistenceunitInfo.getPersistenceProviderClassName() == null
                    || jPersistenceunitInfo.getPersistenceProviderClassName().equals("")) {
                logger.info("No persistence provider was set, set to value {0}.", HIBERNATE_PERSISTENCE_PROVIDER);
                jPersistenceunitInfo.setPersistenceProviderClassName(HIBERNATE_PERSISTENCE_PROVIDER);
            }

            // add JOTM / hibernate property if not set
            if (jPersistenceunitInfo.getProperties().getProperty(HIBERNATE_TM_LOOKUP) == null) {
                jPersistenceunitInfo.getProperties().setProperty(HIBERNATE_TM_LOOKUP, HIBERNATE_JOTM);
            }

        }
        return jPersistenceunitInfos;
    }
}
