/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansResourceAdapter.java 396 2006-04-24 16:35:51Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server.ra;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.server.Embedded;
import org.objectweb.easybeans.server.EmbeddedException;
import org.objectweb.easybeans.server.ServerConfig;

/**
 * Defines a resource adapter which allow to embed the EJB3 server in Application Server.
 * application server.
 * @author Florent Benoit
 */
public class EasyBeansResourceAdapter implements javax.resource.spi.ResourceAdapter {

    /**
     * Embedded instance.
     */
    private Embedded embedded = null;

    /**
     * Starts the embedded server.
     * @param ctx - a bootstrap context containing references to useful
     *        facilities that could be used by a resource adapter instance.
     * @throws ResourceAdapterInternalException indicates bootstrap failure.
     */
    public void start(final BootstrapContext ctx) throws ResourceAdapterInternalException {
        if (embedded == null) {
            ServerConfig serverConfig = new ServerConfig();
            serverConfig.setStartRegistry(false);
            serverConfig.setStartTM(false);
            String jBase = System.getProperty("jonas.base");
            if (jBase == null) {
                throw new ResourceAdapterInternalException("No JONAS_BASE found, cannot continue.");
            }
            File workDir = new File(jBase + File.separator + "ejb3s");
            workDir.mkdir();
            serverConfig.setEjb3Path(workDir.getAbsolutePath());
            serverConfig.setBindDummyDataSourceHSQL(false);
            serverConfig.setShouldWait(false);
            serverConfig.setUseMBeans(false);
            serverConfig.setUseNaming(false);
            embedded = new Embedded(serverConfig);
        }

        // start on the embedded object will be delayed until JOnAS is started.
        // This is because Datasource could be deployed after this adapter and
        // then
        // datasource won't be found.
        new EmbeddedStarter().start();

    }

    /**
     * Stopp the embedded server.
     */
    public void stop() {
        try {
            embedded.stop();
        } catch (EmbeddedException e) {
            throw new IllegalStateException("Cannot stop the embedded server", e);
        } finally {
            embedded = null;
        }
    }

    /**
     * This is called during the activation of a message endpoint.
     * @param endpointFactory - a message endpoint factory instance.
     * @param spec an activation spec JavaBean instance.
     * @throws ResourceException - indicates message endpoint activation
     *         rejection due to incorrect activation setup information.
     */
    public void endpointActivation(final MessageEndpointFactory endpointFactory, final ActivationSpec spec)
            throws ResourceException {

    }

    /**
     * This is called when a message endpoint is deactivated. The instances
     * passed as arguments to this method call should be identical to those
     * passed in for the corresponding endpointActivation call. This causes the
     * resource adapter to stop delivering messages to the message endpoint. Any
     * exception thrown by this method is ignored. After this method call, the
     * endpoint is deemed inactive.
     * @param endpointFactory a message endpoint factory instance.
     * @param spec an activation spec JavaBean instance.
     */
    public void endpointDeactivation(final MessageEndpointFactory endpointFactory, final ActivationSpec spec) {

    }

    /**
     * This method is called by the application server during crash recovery.
     * @param specs an array of ActivationSpec JavaBeans each of which
     *        corresponds to an deployed endpoint application that was active
     *        prior to the system crash.
     * @return an array of XAResource objects each of which represents a unique
     *         resource manager.
     * @throws ResourceException generic exception if operation fails due to an
     *         error condition.
     */
    public XAResource[] getXAResources(final ActivationSpec[] specs) throws ResourceException {

        return null;
    }

    /**
     * Gets the embedded object.
     * @return embedded server.
     */
    protected Embedded getEmbedded() {
        return embedded;
    }

    /**
     * This inner class will wait until the JOnAS server is ready and start the
     * embedded server.
     * @author Florent Benoit
     */
    public class EmbeddedStarter extends Thread {

        /**
         * Sleep time for this waiter thread (1 second).
         */
        private static final int SLEEP_TIME = 1000;

        /**
         * Logger.
         */
        private JLog logger = JLogFactory.getLog(EmbeddedStarter.class);

        /**
         * Wait until JOnAS is started.<br>
         * Then, start embedded object.
         */
        @Override
        public void run() {
            boolean embeddedStarted = false;

            while (!embeddedStarted) {

                try {
                    Thread.sleep(SLEEP_TIME);
                } catch (InterruptedException e) {
                    throw new RuntimeException("Thread fail to sleep");
                }

                if (jonasIsReady()) {
                    try {
                        getEmbedded().start();
                    } catch (EmbeddedException e) {
                        logger.error("Cannot start the embedded server", e);
                    }
                    embeddedStarted = true;
                }
            }
        }

        /**
         *  @return true if the JOnAS server is ready, else false
         */
        private boolean jonasIsReady() {
            String jonasName = System.getProperty("jonas.name", "jonas");

            // Get Adm object
            String admName = jonasName + "_Adm";
            Context initialContext = null;
            try {
                initialContext = new InitialContext();
            } catch (NamingException e) {
                logger.error("Cannot get an initial context", e);
                return false;
            }
            Object adm = null;
            try {
                adm = initialContext.lookup(admName);
            } catch (NamingException e) {
                logger.error("No {0} object found in the initial context", admName, e);
                return false;
                }

            // Try to see if there is a method getServerState on this object
            Method getStateMethod;
            String methodName = "getServerState";
            try {
                getStateMethod = adm.getClass().getMethod(methodName);
            } catch (SecurityException e) {
                logger.error("Cannot get method name {0}", methodName, e);
                return false;
            } catch (NoSuchMethodException e) {
                logger.error("Cannot get method name {0}", methodName, e);
                return false;
            }

            if (getStateMethod == null) {
                logger.warn("No method getStateMethod() found on object {0}", adm);
                return false;
            }

            Integer state = null;
            try {
                state = (Integer) getStateMethod.invoke(adm);
            } catch (IllegalArgumentException e) {
                logger.error("Cannot call method {0} on object {1}.", methodName, adm, e);
                return false;
            } catch (IllegalAccessException e) {
                logger.error("Cannot call method {0} on object {1}.", methodName, adm, e);
                return false;
            } catch (InvocationTargetException e) {
                logger.error("Cannot call method {0} on object {1}.", methodName, adm, e);
                return false;
            }

            // check the status
            if (state.intValue() == 1) {
                logger.info("JOnAS was started, EasyBeans embedded server can be started.");
                // JOnAS is ready, can start embedded !
                return true;
            }

            return false;

        }

    }

}
