/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansContextListener.java 414 2006-04-26 20:45:12Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server.war;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.server.Embedded;
import org.objectweb.easybeans.server.EmbeddedException;
import org.objectweb.easybeans.server.ServerConfig;

/**
 * Listener class which is notified of lifecycle events. It allows to start
 * EasyBeans at startup and stop it at the end.
 * @author Florent Benoit
 */
public class EasyBeansContextListener implements ServletContextListener {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(EasyBeansContextListener.class);

    /**
     * Embedded instance.
     */
    private Embedded embedded = null;

    /**
     * List of available supported container.
     */
    private enum AvailableWebContainer {
        /**
         * Containers supported.
         */
        TOMCAT, JETTY, UNKNOWN
    }

    /**
     * Container detected ?.
     */
    private AvailableWebContainer containerType;

    /**
     * Notification that the web application initialization process is starting.
     * All ServletContextListeners are notified of context initialization before
     * any filter or servlet in the web application is initialized.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    public void contextInitialized(final ServletContextEvent servletContextEvent) {

        // Create and configure EasyBeans embedded server
        if (embedded == null) {
            ServerConfig serverConfig = new ServerConfig();
            serverConfig.setStartRegistry(true);
            serverConfig.setStartTM(true);
            serverConfig.setBindDummyDataSourceHSQL(true);
            serverConfig.setShouldWait(false);
            serverConfig.setUseMBeans(true);
            serverConfig.setUseNaming(false);
            // Detects Tomcat/Jetty/etc.
            String ejb3ParentDir = detectWebContainerDirectory();
            logger.info("Detecting '" + containerType + "' web container");
            // Create ejb3s directory
            File workDir = new File(ejb3ParentDir + File.separator + "ejb3s");
            workDir.mkdir();
            serverConfig.setEjb3Path(workDir.getAbsolutePath());
            embedded = new Embedded(serverConfig);
            registerLibraries(servletContextEvent);
            try {
                embedded.start();
            } catch (EmbeddedException e) {
                throw new IllegalStateException("Cannot start embedded EasyBeans server", e);
            }
        }
    }

    /**
     * Notification that the servlet context is about to be shut down. All
     * servlets and filters have been destroy()ed before any
     * ServletContextListeners are notified of context destruction.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    public void contextDestroyed(final ServletContextEvent servletContextEvent) {
        if (embedded != null) {
            logger.info("Stopping EasyBeans embedded server...");
            try {
                embedded.stop();
            } catch (EmbeddedException e) {
                throw new IllegalStateException("Cannot stop the embedded server", e);
            } finally {
                embedded = null;
            }
        }
    }

    /**
     * Gets the directory for EJB3 depending on the servlet container.
     * @return path to catalina.base/jetty.home/etc.
     */
    private String detectWebContainerDirectory() {
        // catalina ?
        String catalinaBase = System.getProperty("catalina.base");
        if (catalinaBase != null) {
            containerType = AvailableWebContainer.TOMCAT;
            return catalinaBase;
        }

        // Jetty ?
        String jettyHome = System.getProperty("jetty.home");
        if (jettyHome != null) {
            containerType = AvailableWebContainer.JETTY;
            return jettyHome;
        }

        // not found, return tmp directory
        containerType = AvailableWebContainer.UNKNOWN;
        return System.getProperty("java.io.tmpdir");
    }

    /**
     * Register the EasyBeans libraries in the root classloader (allowing each
     * applications to communicate with EasyBeans).
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    private void registerLibraries(final ServletContextEvent servletContextEvent) {
        if (containerType.equals(AvailableWebContainer.TOMCAT)) {
            registerTomcat(servletContextEvent);
        }

    }

    /**
     * Register the EasyBeans libraries for Tomcat.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    private void registerTomcat(final ServletContextEvent servletContextEvent) {

        // Current Classloader
        ClassLoader currentCL = Thread.currentThread().getContextClassLoader();

        // Now, search shared classloader (child of server CL)
        // Assume that it is a correct Tomcat installation
        ClassLoader cl = currentCL.getParent().getParent();
        // Check class :
        if (!"org.apache.catalina.loader.StandardClassLoader".equals(cl.getClass().getName())) {
            throw new IllegalStateException("Didn't find common classloader");
        }

        // Cast classLoader in URL Classloader
        URLClassLoader urlCl = (URLClassLoader) cl;
        // Get addURL method
        Method addURL = null;
        try {
            addURL = URLClassLoader.class.getDeclaredMethod("addURL", new Class[] {URL.class});
        } catch (SecurityException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        addURL.setAccessible(true);

        // Add libraries
        // / Get Carol jar
        ServletContext sContext = servletContextEvent.getServletContext();
        List<String> libraries = new ArrayList<String>();
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ow_carol.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ow_carol_cmi.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ow_ejb3_api.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ejb-2_1-api.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ejb3-persistence.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ow_easybeans_api.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/ow_easybeans_core.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/jotm.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/jotm_jrmp_stubs.jar"));
        libraries.add(sContext.getRealPath("/WEB-INF/lib/jta-spec1_0_1.jar"));

        for (String library : libraries) {
            URL url = null;
            try {
                url = new File(library).toURL();
            } catch (MalformedURLException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
            try {
                logger.info("Registering '" + url + "'.");
                addURL.invoke(urlCl, url);
            } catch (IllegalArgumentException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        addURL.setAccessible(false);

    }
}
