/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DBConnectionManager.java 363 2006-04-19 14:40:11Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.db;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.sql.DataSource;

/**
 * Used to gets and manages the datasource.
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class DBConnectionManager implements Serializable {

    /**
     * The serial version.
     */
    private static final long serialVersionUID = -4370744738213787351L;

    /**
     * The database used to insert the tables.
     */
    private DataSource dataSource = null;

    /**
     * The connection with the database.
     */
    private Connection dbConnection = null;

    /**
     * Default Constructor.
     *
     */
    public DBConnectionManager(){

    }

    /**
     * Constructor that receives the DataSource.
     * @param ds datasource to manage.
     */
    public DBConnectionManager(final DataSource ds){
        if (ds == null){
            throw new IllegalStateException("The datasource must not be null.");
        }
        setDataSource(ds);
    }
    /**
     * Returns the instance datasource .
     * @return the datasource
     */
    public DataSource getDataSource() {
        return dataSource;

    }

    /**
     * Sets a datasource if the caller has already an instance of datasource.
     * @param dtSource the datasource added
     */
    public void setDataSource(final DataSource dtSource) {
        dataSource = dtSource;
    }

    /**
     * Makes an update in the database.
     * @param sqlQuery the sql query to be inserted
     * @throws SQLException if a database error occurs
     */
    public void updateDB(final String sqlQuery) throws SQLException {
         PreparedStatement stmUpdate = null;
        try {
            stmUpdate = dbConnection.prepareStatement(sqlQuery);
            stmUpdate.executeUpdate();
        } finally {
            if (stmUpdate != null) {
                stmUpdate.close();
            }
       }
    }

    /**
     * Returns the preparedStatement.
     * @param sqlQuery an SQL statement that may contain one or more '?'.
       @return the preparetedStatement.
     * @throws SQLException if a database error occurs
     */
    public PreparedStatement getPreparedStatement(final String sqlQuery) throws SQLException {
        return dbConnection.prepareStatement(sqlQuery);
    }

    /**
     * Creates a connection with the database.
     * @throws SQLException if a database error occurs
     */
    public void connect() throws SQLException {
        dbConnection = dataSource.getConnection();
    }

    /**
     * Closes the connection.
     * @throws SQLException if a database error occurs.
     */
    public void close() throws SQLException {
        dbConnection.close();
    }

}
