/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TableManager.java 389 2006-04-24 07:23:54Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.db;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.naming.NamingException;
import javax.sql.DataSource;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.server.Embedded;
import org.objectweb.easybeans.tests.common.helper.DBHelper;

/**
 * Manages a table in the database.
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class TableManager {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Embedded.class);

    /**
     * Connection with the database.
     */
    private DBConnectionManager dbConnection = null;

    /**
     * Creates a new instance of TableManager.
     * @param dbConnection the connection with the database.
     */
    public TableManager(final DBConnectionManager dbConnection) {
        this.dbConnection = dbConnection;
    }

    /**
     * Creates a new instance of TableManager.
     * @param dbName the database name in the registry.
     * @throws NamingException if a lookup error occurs.
     */
    public TableManager(final String dbName) throws NamingException {
        dbConnection = new DBConnectionManager(DBHelper.getDataSource(dbName));
    }

    /**
     * Creates a new instance of TableManager.
     * @param ds the datasource.
     */
    public TableManager(final DataSource ds) {
        dbConnection = new DBConnectionManager(ds);
    }

    /**
     * Creates the table in the database done without make the commit after the
     * insertion.
     * @param tableName the table name.
     * @throws SQLException if a database error occurs.
     */
    public void insertTable(final String tableName) throws SQLException {
        try {
            logger.debug("Before insert table.");
            dbConnection.connect();
            logger.debug("Connection opened.");
            dbConnection.updateDB("CREATE TABLE " + tableName + " (CodeTest integer, NameTest varchar(30), "
                    + "PRIMARY KEY (CodeTest))");
            logger.debug("Table created.");
        } catch(Exception e){
            logger.debug("Exception inserting table: {0}", e.getMessage());
        } finally {
            if (dbConnection != null) {
                dbConnection.close();
                logger.debug("Connection closed.");
            }
        }
    }

    /**
     * Deletes the table test. This methods does not make the commit after
     * delete.
     * @param tableName the table name.
     * @throws SQLException if a database error occurs.
     */
    public void deleteTable(final String tableName) throws SQLException {
        try {
            dbConnection.connect();
            // deletes the table
            dbConnection.updateDB("DROP TABLE " + tableName + " CASCADE");
            dbConnection.close();
        } catch(Exception e){
            logger.debug("Exception deleting table: {0}", e.getMessage());
        } finally {
            if (dbConnection != null) {
                dbConnection.close();
            }
        }
    }

    /**
     * Verifies if the table was created.
     * @param tableName the table name.
     * @throws SQLException if a database error occurs or the table not exists.
     */
    public void verifyTable(final String tableName) throws SQLException {
        try {
            dbConnection.connect();
            PreparedStatement stmt = null;
            try {
                stmt = dbConnection.getPreparedStatement("SELECT * FROM " + tableName);
                stmt.execute();
            } catch(Exception e){
                logger.debug("Exception verifying table: {0}", e.getMessage());
            } finally {
                if(stmt != null){
                    stmt.close();
                }
            }
        }  catch(Exception e){
            logger.debug("Exception verifying table: {0}", e.getMessage());
        }finally {
            if (dbConnection != null) {
                dbConnection.close();
            }
        }
        dbConnection.close();
    }

    /**
     * Creates and deletes a table.
     * @param tableName the table name.
     * @throws SQLException if a database error occurs or the table not exists.
     */
    public void test(final String tableName) throws SQLException {
        insertTable(tableName);
        deleteTable(tableName);
    }
}
