/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EBaseInterceptorAccess01.java 363 2006-04-19 14:40:11Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.ejbs.base;

import static org.testng.Assert.assertTrue;

import javax.annotation.Resource;
import javax.ejb.AroundInvoke;
import javax.ejb.EJB;
import javax.ejb.InvocationContext;
import javax.ejb.SessionContext;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;
import javax.sql.DataSource;

import org.objectweb.easybeans.tests.common.db.TableManager;
import org.objectweb.easybeans.tests.common.ejbs.entity.ebstore.EBStore;
import org.objectweb.easybeans.tests.common.helper.JNDIHelper;
import org.objectweb.easybeans.tests.common.resources.EMFactoryTester;
import org.objectweb.easybeans.tests.common.resources.EntityManagerTester;


/**
 * Used to test the interceptors access to the following resources.<br> <li>SessionContext
 * Methods</li> <li>JNDI Access to java:comp/env</li> <li>Resource Manager</li>
 * <li>Enterprise bean</li> <li>EntityManagerFactory</li> <li>EntityManage</li>
 * <li>TimerService</li> <li>UserTransaction</li>
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public class EBaseInterceptorAccess01 implements ItfAccessJNDI, ItfAccessEJB, ItfAccessResourceManager,
        ItfAccessEntityManager, ItfAccessEMFactory, ItfAccessSessionContext{

    /**
     * Bean.
     */
    @EJB(name="ejb/bean", beanName="EJBInjectionBean")
    private ItfOneMethod01 bean;

    /**
     * Injected DataSource.
     */
    @Resource(name = "jdbc/jdbc_1", mappedName="jdbc_1")
    private DataSource ds;

    /**
     * Entity Manager.
     */
    @PersistenceContext(unitName="testEntity00")
    private EntityManager eManager = null;

    /**
     * Entity Factory.
     */
    @PersistenceUnit(unitName="testEntity00")
    private EntityManagerFactory entityFactory = null;

    /**
     * SessionContext.
     */
    @Resource
    private SessionContext ctx;

    /**
     * This has an interceptor that accesses the java:comp/env via JNDI.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessJNDI(final Object obj) throws Exception {
        return null;
    }

    /**
     * This has an interceptor that accesses an EJB.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessEJB(final Object obj) throws Exception {
        return null;
    }

    /**
     * This has an interceptor that accesses a resource.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessResManager(final Object obj) throws Exception {
        return null;
    }

    /**
     * This has an interceptor that accesses an entity manager.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessEntityManager(final Object obj) throws Exception {
        return null;
    }

    /**
     * This has an interceptor that accesses an entity manager.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessEntityManagerFactory(final Object obj) throws Exception {
        return null;
    }

    /**
     * This has an interceptor that accesses the sessionContext and try to lookup a datasource.
     * @param obj it's not used.
     * @return null
     * @throws Exception if a problem occurs.
     */
    public Object accessSessionContext(final Object obj) throws Exception {
        return null;
    }

    /**
     * Business Method Inteceptor that accesses the resource specified by the class comments.
     * @param ic contains attributes of invocation..
     * @return method's invocation result
     * @throws Exception if invocation fails
     */
    @AroundInvoke
    public Object intercept(final InvocationContext ic) throws Exception{
        if (ic.getMethod().toString().contains("accessJNDI")) {

            //JNDI Access
            DataSource dsViaJNDI = (DataSource) JNDIHelper.getJavaCompEnvResource("jdbc/jdbc_1");
            TableManager cTest = new TableManager(dsViaJNDI);
            cTest.test(Integer.toString(cTest.hashCode()));

        } else if (ic.getMethod().toString().contains("accessEJB")) {

            //EJB Access
            assertTrue(bean.getBool());

        } else if (ic.getMethod().toString().contains("accessResManager")) {

            //ResourceManager Access
            TableManager cTest = new TableManager(ds);
            cTest.test(Integer.toString(cTest.hashCode()));

        } else if (ic.getMethod().toString().contains("accessEntityManager")) {

            //EntityManager Access
            doTransaction(eManager, EntityManagerTester.NAME);


        } else if (ic.getMethod().toString().contains("accessEntityManagerFactory")) {

            //EntityManagerFactory Access
            EntityManager entityManager = entityFactory.createEntityManager();
            doTransaction(entityManager, EMFactoryTester.NAME);

        } else if (ic.getMethod().toString().contains("accessSessionContext")) {

            //SessionContext Access
            if (ctx == null){
                throw new Exception("SessionContext is null.");
            }

        } else{
            throw new Exception("Invalid method to intercept.");
        }
        return ic.proceed();
    }

    /**
     * Does a transaction operation.
     * @param entityManager EntityManager
     * @param name Name
     * @throws Exception if a problem occurs
     */
    private void doTransaction(final EntityManager entityManager, final String name) throws Exception {
        EBStore store = new EBStore();
        store.setId(store.hashCode());
        store.setName(name);
        entityManager.persist(store);

        if (entityManager.find(EBStore.class, Integer.valueOf(store.hashCode())) == null){
            throw new Exception("EBStore was created, but it wasn't found after the commit.");
        }
    }
}
